/*
 *   This file is part of Auralquiz
 *   Copyright 2011-2017  JanKusanagi JRR <jancoding@gmx.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .
 */

#ifndef ANSWERBOX_H
#define ANSWERBOX_H


#include <QLineEdit>
#include <QString>
#include <QRegExp>
#include <QDebug>


class AnswerBox : public QLineEdit
{
    Q_OBJECT

public:
    AnswerBox(QWidget *parent = 0);
    ~AnswerBox();

    QString normalizeString(QString original);
    void setAnswer(QString newAnswer);

public slots:
    void validateAnswer();

signals:
    void answered(bool correct);

private:
    QString correctAnswer;


};

#endif // ANSWERBOX_H
